% This Script was developed to find the paternal pronuclei in C. elegans
% embryos for use in the Bahmanyar Lab at Yale University. This was
% developed by Michael S. Mauro. 

dbstop if error;
New_Path = '/Users/michaelmauro/Documents/MATLAB/MAX_C2-1-3-18OD270lpin-1rnai012_Cropped_MATLAB'; % This will automatically set the right directory
userpath(New_Path);
myFolder = '/Users/michaelmauro/Documents/MATLAB/MAX_C2-1-3-18OD270lpin-1rnai012_Cropped_MATLAB';

% To adjust this you just have to change the route to where ever your
% folder of interest is located. This is the directory from which the
% script will pull the files from to analyze. Only put in the specific
% frames that you want to get data for.

%this next section will just tell you if you made a mistake with the
%folder's location, always check the spelling and spaces in the myfolder
%pathway. MATLAB is case and space sensitive. Always check that there is no
%extra space at the end of variable for example, which can sometimes be
%missed. 


if ~isdir(myFolder) 
  E= errordlg('Error: The following folder does not exisst','File Path Error'); 
  waitfor(E);
end


% Get a list of all files in the folder with the desired file name pattern .
% Change to whatever pattern you need so like tiff or any other file type.

filePattern = fullfile(myFolder, '*.jpg'); % This will tell you how many files are in the folder with the certain type.
theFiles = dir(filePattern);
for k = 1 : length(theFiles) 
   
end

%This next section is where you tell the script where to start and stop,
% if you wish to do all the files then start = 1 and stop =k which is the
% full amount of files.
Start=1; % whichever file you want to start at.
Stop=  k;  % whichever file you want to stop at.

% Before you go into the full loop you need to look at the pixel intesity
% of the first image to get a sense for what you need to set the threshold
% value for to eliminate most of the background. The hope is that you set
% this once and then all the rest of the images in the video are ok because
% they were all processed the same.


for i=Start:Start
    n=theFiles(i).name;
    Original_Image=imread(n);
    figure;
    set(gcf,'numbertitle','off','name', 'Original Image & Histogram'); 
    subplot(2,1,1)
    H= imshow(Original_Image);
    title('Original Image');    

    
    subplot(2,1,2);
    [pixelCount] = imhist(Original_Image);   
    bar(pixelCount,'k');
    axis([0 80 0 1000])
    title('Original Image Histogram');
    xlabel('Pixel Intensity','FontSize',16);
    ylabel('Number of Pixels','FontSize',16);
    grid on;
    waitfor(H);
   
    
    
    prompt = {'Enter Threshold Value:'};
    dlg_title = 'Please Input Values';
    num_lines = 1;
    defaultans = {'25'};
    answer = inputdlg(prompt,dlg_title,num_lines,defaultans);
    Answer = cell2mat(answer(:));
    Answer = str2double(Answer);
    Threshold_Value = Answer;
    
    H= figure;  
    set(gcf,'numbertitle','off','name', 'Thresholded Histogram & Binary Image'); 
    subplot(2,1,1);
    [pixelCount] = imhist(Original_Image);
    bar(pixelCount,'k');
% Scale x axis manually. You just want to be able to see most of the pixel
% values clearly.
    axis([0 80 0 1000])
    hold on
    plot([Threshold_Value,Threshold_Value],[0,1000], 'r')
    T1=text(Threshold_Value + 4, 500, 'Threshold Value');
    T1(1).FontSize = 18;
    T1(1).Color = 'red';
    xlabel('Pixel Intensity','FontSize',16);
    ylabel('Number of Pixels','FontSize',16);
    title('Original Image Histogram With Threshold Value');    
    hold off

    grid on;
    
%The next section of code will create a binary image, turning everything
%into a 1 if above the threshold value. 1 is white and 0 is black
    subplot(2,1,2);
    Binary_Image = Original_Image > Threshold_Value; 
    Binary_Image = imfill(Binary_Image, 'holes'); %this will fill any holes present     
    imshow(Binary_Image);
    title('Binary Image Based on Threshold');    
end


waitfor(H);

% The button command allows you to ask a question and depending on the
% answer choosen will cause different things to occur. I set this up so
% that one could cancel the run if they didnt like the threshold that they
% set up. 

button = questdlg('Do you wish to continue?', 'Attention', 'Yes', 'No','Help','Help');

 if strcmp(button,'Yes')==1
   
    for i=Start:Stop
        n=theFiles(i).name;
        Original_Image=imread(n);
        Binary_Image = Original_Image > Threshold_Value; % Generates a binary image
        Binary_Image = imfill(Binary_Image, 'holes'); % This will fill any holes present.
        Labeled_Image = bwlabel(Binary_Image);% Label each blob so we can make measurements of it.
        Labeled_Image = bwareaopen(Labeled_Image,25);%Removes anything with an area smaller than the pixel value indicated, should help with background.
        Labeled_Image = bwlabel(Labeled_Image); % This will group together the various blobs
        %imshow(Labeled_Image);
        % The next few sections are to remove the other nuclei and polar
        % bodies. Essentially everything you dont want. This can be
        %adjusted so you can exclude just the polar bodies
        
        A= regionprops(Labeled_Image, 'all'); %#ok<*MRPBW> % This will do a quick run through of region props to get data for each blob, this will be used to remove blobs other than paternal.
        B= regionprops(Labeled_Image, Original_Image, 'Meanintensity','MinIntensity'); % This will give data that will help to remove the polar bodies specifically, which have a high level pixel intensity.
        
        % This next section will calculate the distance between a mid point
        % on the left side, to each blob. I devised this as a way to ensure
        % that you will only keep the paternal pronuclei, which is always
        % the furthest to the right.
        Number_Of_Blobs =size(A,1); % This will tell you the number of blobs. 
        Point_Y1= size(Labeled_Image); %Label a point that is halfway up the image. 
        Point_Y1= Point_Y1(1,1)/2; %Label a point that is halfway up the image. 
        Point_X1 =10; %Label a point that is 10 pixels from the left of the image. 
        %H= figure;
        %imshow(Labeled_Image); %This is optional code to plot the above mentioned point
%         hold on
% 
%         plot(Point_X1, Point_Y1,'m+','Color','r');
% 
%         hold off
        
       for  i2 = 1 : Number_Of_Blobs;
            Distance(i2) = sqrt((A(i2).Centroid(1)-Point_X1)^2+(A(i2).Centroid(2)-Point_Y1)^2); %#ok<*SAGROW>
       end
       
       All_Blob_Eccentricity = [A.Eccentricity]; % This extracts all the eccentricity values. 
       All_Blob_Mean_Intensity = [B.MeanIntensity];% This extracts all the mean intensity values. 
       All_Blob_Distance = Distance; % This exracts all the distance values.
       Distance_Factor= max(All_Blob_Distance)-1; % This sets up the threshold, its essentially the farthest point -1 so it will include just the paternal.
       Allowable_Eccentricity_Indexes = (All_Blob_Eccentricity > .05 & All_Blob_Eccentricity <.95 ); % this is setting up the values to keep for eccentricity.
       Allowable_Mean_Intensity = (All_Blob_Mean_Intensity > 13 & All_Blob_Mean_Intensity < 200); % this is setting up the values to keep for intensity.
       Allowable_Distance= All_Blob_Distance > Distance_Factor; % this is setting up the values to keep for distance.
       Keeper_Factors = find(Allowable_Eccentricity_Indexes & Allowable_Mean_Intensity & Allowable_Distance); % This is all three put together to esentially remove everything but the paternal.
       Keeper_Blobs_Image= ismember(Labeled_Image, Keeper_Factors); %This actually collects the keeper image, the ismember allows you to do this. 
       % imshow(Keeper_Blobs_Image); % This shows that keeper image
       
       %This next section will generate the boundaries of the blob, which
       %can be plotted on to both the original image and binary to show how
       %accurate it is. Ideally this should be a pretty close fit over the
       %original image. If its not you will need to mess around with the
       %pixel threshold value. 
       Boundaries = bwboundaries(Keeper_Blobs_Image);
       Number_Of_Boundaries = size(Boundaries, 1);
       for i3 = 1 : Number_Of_Boundaries
           This_Boundary = Boundaries{i3};
       end
% This next section will get all the information about the nuclei using the
% regionprops command. All of this information is in pixel value. This will
% be saved after each loop to ensure that data is all kept.
    E=0;
    if Number_Of_Boundaries>E
    S(i)= regionprops(Keeper_Blobs_Image, 'Centroid','Area','Eccentricity','EquivDiameter', 'MajorAxisLength', 'MinorAxisLength');
    
% This next chunk of code will basically extract all of the data and also
% include the name of the file that it corresponds to. The pixel values
% will also be converted to microns using the Objective pixel/micron scale.
% in the case of the Bahmanyar Lab its .23 micron/pixel. The centroid is
% also split up into the X and Y coordinates. See 
%http://www.mathworks.com/help/images/ref/regionprops.html for more
%information about all the various components.

    Blob_Measurements{i,1}= n;  
    Blob_Measurements{i,2}=  S(i).EquivDiameter*.23;
    Blob_Measurements{i,3}=  (S(i).EquivDiameter*.23)/2; %radius
    Blob_Measurements{i,4}=  (((S(i).EquivDiameter*.23)/2)^2)*4*pi; %surface area
    Blob_Measurements{i,5}=  (((S(i).EquivDiameter*.23)/2)^3)*(4/3)*pi; %Volume
    Blob_Measurements{i,6}=  S(i).EquivDiameter;
    Blob_Measurements{i,7}=  S(i).Eccentricity;
    Blob_Measurements{i,8}=  S(i).MajorAxisLength*.23;
    Blob_Measurements{i,9}=  S(i).MajorAxisLength;
    Blob_Measurements{i,10}=  S(i).MinorAxisLength*.23;
    Blob_Measurements{i,11}=  S(i).MinorAxisLength;    
    Blob_Measurements{i,12}=  S(i).MajorAxisLength/ S(i).MinorAxisLength;  
    Blob_Measurements{i,13}= S(i).Centroid(1);
    Blob_Measurements{i,14}= S(i).Centroid(2);
    Blob_Measurements{i,15}= S(i).Area*.0529; %uses the pixel area
    Blob_Measurements{i,16}= S(i).Area;
    Blob_Measurements{i,17}= Threshold_Value;
  
%This next chunk of code will create a figure with multiple plots that has
%the original image, the image with the boundaries of the binary graphed on
%it, the image with a circle on it, and the image with both boundaries and
%the circle. This should be saved in a subfolder called 'Figures'. This can
%be looked at later to make sure that there was a good conversion of the
%original image to binary and that the data that regionprops collected was
%accurate. 
    set(gcf,'numbertitle','off','name',['Figure For: ', num2str(n)]) % The num2str names the figure whatever file you are currently on. 
    subplot(2,2,1);   
        imshow(Original_Image);
        title('Original image');
    subplot(2,2,2);
        imshow(Original_Image);
        title('Binary boundaries');
        hold on
        % This will plot the boundaries of the binary image onto the
        % original image.
        for k = 1 : Number_Of_Boundaries 
        This_Boundary = Boundaries{k};
        plot(This_Boundary(:,2), This_Boundary(:,1), 'r', 'LineWidth', 1,'Color','b');
        end
        hold off  
    
    subplot(2,2,3);
        imshow(Original_Image);
        title('Corresponding circle');     
        hold on
        % This plots the circle using the equivdiameter determined by regionprops. 
        for k = 1 : Number_Of_Blobs;  
        radius = S(i).EquivDiameter/2; 
        centerX = S(i).Centroid(1);
        centerY = S(i).Centroid(2);
        viscircles([centerX, centerY], radius,'LineWidth', 1,'LineStyle','-','Color','r');
        %plot(centerX,centerY,'m+','Color','r');
        end
        hold off
        
    subplot(2,2,4);
        imshow(Original_Image);
        title('Boundaries & corresponding circle'); 
        hold on
        % This plots both at the same time.   
        for k = 1 : Number_Of_Blobs;   
        radius = S(i).EquivDiameter/2; 
        centerX = S(i).Centroid(1);
        centerY = S(i).Centroid(2);
        viscircles([centerX, centerY], radius,'LineWidth', 1,'LineStyle','--','Color','r');
        plot(centerX,centerY,'m+','Color','r');
        end
        for k = 1 : Number_Of_Boundaries
        This_Boundary = Boundaries{k};
        plot(This_Boundary(:,2), This_Boundary(:,1), 'r', 'LineWidth', 1,'Color','b');
        end
        hold off
    
% This will save the figures in the subfolder called Figures. You need to
% make sure that the folder is just one level past your current directory
% so it gets saved in the right spot. These individual figures can be
% spliced together in FIJI to make a video of the plotting information. 
%     fname = 'Figures/';
%     saveas(gcf,fullfile(fname,['Figure For: ', num2str(n)]));
%     pause('on')  
%     n3 = .5;
    % pause(n3) 
    clearvars -except myFolder filePattern theFiles Start Stop S Threshold_Value Blob_Measurements n % You need to clear the variables, but these essential ones so that way it runs through the loop smoothly. 
    elseif Number_Of_Boundaries==E 
    S(i)=struct('Centroid',{'0'},'Area',{'1'},'Eccentricity',{'1'},'EquivDiameter',{'1'},'MajorAxisLength',{'1'},'MinorAxisLength',{'1'});
    set(gcf,'numbertitle','off','name',['Figure For: ', num2str(n)]) % The num2str names the figure whatever file you are currently on. 
    subplot(2,2,1);   
        imshow(Original_Image);
        title('Original image');
    subplot(2,2,2);
        imshow(Original_Image);
        title('Original image');
         
    
    subplot(2,2,3);
        imshow(Original_Image);
        title('Original image');     
            
    subplot(2,2,4);
        imshow(Original_Image);
        title('Original image');
    
    end
    fname = 'Figures/';
    saveas(gcf,fullfile(fname,['Figure For: ', num2str(n)]));
%     pause('on')  
%     n3 = .5;
end
% These next few lines generate a table, which can be converted back into a
% cell array that has the top of each column with the right label of what
% that specific variable is. 
C = cell2table(Blob_Measurements,'VariableNames',{'File_Name','Real_EquivDiameter','Real_Radius','Real_Surface_Area','Real_Volume','Pixel_EquivDiameter','Eccentricity','Real_MajorAxisLength','Pixel_MajorAxisLength','Real_MinorAxisLength','Pixel_MinorAxisLength','Major_Minor_Ratio','Centroid_X','Centroid_Y','Real_Area','Pixel_Area','Threshold_Value'});
D = table2cell(C);
Paternal_Pronuclei_Data= [C.Properties.VariableNames;D];
     
elseif strcmp(button,'No')==1
    h = msgbox('Please run script again with different threshold value. To get more pixels in the binary image set a lower threshold. For less pixels converted, set higher. ','Help Box','help');   
elseif strcmp(button,'Help')==1
    h = msgbox('Threshold value is based upon the histogram, choose a value that eliminates most pixels, so that your area of interest is well segmented. The key here is to set it low enough to get through all the images but high enough to prevent background','Help Box','help');   
end
F = msgbox('Run Completed');

 
%% This section will perform a linear regression on the data, you just simply have to indicate where to start after the lag for expansion.
Y= Blob_Measurements(1:end,2); % Takes out just the diameter in microns so that it can be plotted against time.
W= size(Y,1); % This will give the number of time points.
Y=cell2mat(Y); % This converts it to be able to plot it. 
X=0:20:((W-1)*20); % This makes the actual X axis in seconds, because the first time point is 0 its actually 10 seconds less, hence the W-1.
X=X'; % This converts the X it a different size matrix so it can be plotted. 
% 
% [betahats,R2,fits,resids,tstats,varcovbetas,pvalues,sebetas]=mvols(Y,X); % This Linear regression function was written by a mathematician/economist. 
% 
% Linear_Regression_Line = betahats(2)*(X)+betahats(1);

Name= n(5:25);% This will come up with a name based on the characters that are present in the variable n, this would have to change if the name is in a different format

figure;
plot(X,Y,'b');
% % hold on
% % plot(X,Linear_Regression_Line,'r');
% % 
% plot(X,Y);
% R2_Value=text(20, 5.5, ['R^2 = ', num2str(R2)]);
% R2_Value(1).FontSize = 18;
% R2_Value(1).Color = 'red';
xlabel('Time (seconds)', 'FontSize',16);
ylabel('Nucleus Diameter (micrometers)','FontSize',16);
title(['Individual trace for Paternal nuclei ', num2str(Name)],'FontSize',16); 
L= legend('Observed data', 'location','southeast',16);
set(L,'FontSize',14);
% hold off
% 
 
fname = 'Paternal Nuclei Traces/';
saveas(gcf,fullfile(fname,[num2str(Name), '_Trace']),'fig');
fname = 'Paternal Nuclei Traces/';
saveas(gcf,fullfile(fname,[num2str(Name), '_Trace']),'jpg');



